package com.yonge.cooleshow.biz.dal.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.biz.dal.entity.*;
import com.yonge.cooleshow.biz.dal.enums.GoodTypeEnum;
import com.yonge.cooleshow.biz.dal.enums.OrderStatusEnum;
import com.yonge.cooleshow.biz.dal.enums.SourceTypeEnum;
import com.yonge.cooleshow.biz.dal.mapper.TenantAlbumMapper;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.vo.UserOrderDetailVo;
import com.yonge.cooleshow.biz.dal.wrapper.TenantAlbumWrapper;
import com.yonge.cooleshow.biz.dal.wrapper.UserPaymentOrderWrapper;
import com.yonge.cooleshow.common.enums.EActivationCode;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.toolset.base.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 机构专辑
 * 2023-07-21 17:32:49
 */
@Slf4j
@Service
public class TenantAlbumServiceImpl extends ServiceImpl<TenantAlbumMapper, TenantAlbum> implements TenantAlbumService {

    @Autowired
    private StudentService studentService;

    @Autowired
    private TenantStaffService tenantStaffService;

    @Autowired
    private TenantInfoService tenantInfoService;

    @Autowired
    private TenantAlbumRefService tenantAlbumRefService;

    @Autowired
    private UserOrderService userOrderService;

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private TenantAlbumPurchaseService tenantAlbumPurchaseService;

    @Autowired
    private TenantAlbumMusicService tenantAlbumMusicService;

    @Autowired
    private TenantActivationCodeService tenantActivationCodeService;

    @Autowired
    private RedisCacheService redisCacheService;

    @Autowired
    private TeacherService teacherService;

    /**
     * 查询详情
     *
     * @param id 详情ID
     * @return TenantAlbum
     */
    @Override
    public TenantAlbum detail(Long id) {

        return baseMapper.selectById(id);
    }

    /**
     * 分页查询
     *
     * @param page  IPage<TenantAlbum>
     * @param query TenantAlbumWrapper.TenantAlbumQuery
     * @return IPage<TenantAlbum>
     */
    @Override
    public IPage<TenantAlbumWrapper.TenantAlbum> selectPage(IPage<TenantAlbumWrapper.TenantAlbum> page,
                                                            TenantAlbumWrapper.TenantAlbumQuery query) {
        return page.setRecords(baseMapper.selectPage(page, query));
    }

    /**
     * 添加
     *
     * @param tenantAlbum TenantAlbumWrapper.TenantAlbum
     * @return Boolean
     */
    @Override
    public Boolean add(TenantAlbum tenantAlbum) {

        //return this.save(JSON.parseObject(tenantAlbum.jsonString(), TenantAlbum.class));
        return this.save(tenantAlbum);
    }

    /**
     * 更新
     *
     * @param tenantAlbum TenantAlbumWrapper.TenantAlbum
     * @return Boolean
     */
    @Override
    public Boolean update(TenantAlbumWrapper.TenantAlbum tenantAlbum) {

        return this.updateById(JSON.parseObject(tenantAlbum.jsonString(), TenantAlbum.class));
    }

    /**
     * 订单创建前检测
     *
     * @param orderGoodsInfo
     */
    @Override
    public void orderCreate(UserPaymentOrderWrapper.OrderGoodsInfo orderGoodsInfo) {
        log.info("订单创建前检测");
        if (orderGoodsInfo == null) {
            log.error("订单创建前检测，订单商品信息为空");
            throw new BizException("订单商品信息为空");
        }


        TenantAlbumWrapper.TenantAlbumContent tenantAlbumContent = JSON
                .parseObject(JSON.toJSONString(orderGoodsInfo.getBizContent()), TenantAlbumWrapper.TenantAlbumContent.class);
        if (tenantAlbumContent == null) {
            log.error("订单创建前检测，订单商品信息为空");
            throw new BizException("订单商品信息为空");
        }


        // 专辑是否存在
        TenantAlbum tenantAlbum = this.getById(orderGoodsInfo.getBizId());
        if (tenantAlbum == null || tenantAlbum.getDelFlag() || Boolean.FALSE.equals(tenantAlbum.getStatus())) {
            log.error("订单创建前检测，专辑不存在，bizId={}", orderGoodsInfo.getBizId());
            throw new BizException("订单商品信息为空");
        }
        tenantAlbumContent.setBuyCycle(tenantAlbum.getPurchaseCycle());
        tenantAlbumContent.setSalePrice(tenantAlbum.getSalePrice());
        tenantAlbumContent.setOriginalPrice(tenantAlbum.getOriginalPrice());

        // 查询专辑所在机构
        List<TenantAlbumRef> refList =
                tenantAlbumRefService.getByAlbumId(tenantAlbum.getId());
        if (CollectionUtils.isEmpty(refList)) {
            log.error("订单创建前检测，专辑不在机构下，bizId={}", orderGoodsInfo.getBizId());
            throw new BizException("用户不在当前机构");
        }

        // 机构ID集合
        List<Long> tenantIdList =
                refList.stream().map(TenantAlbumRef::getTenantId).collect(Collectors.toList());

        UserOrderDetail userOrderDetail = JSON.parseObject(JSON.toJSONString(orderGoodsInfo), UserOrderDetail.class);

        BigDecimal price = BigDecimal.ZERO;
        // 学生购买机构专辑
        switch (orderGoodsInfo.getPaymentClient()) {
            case STUDENT: {
                // 判断学生是否在当前专辑机构下
                Student student = studentService.getById(orderGoodsInfo.getUserId());
                if (student == null) {
                    log.error("订单创建前检测，学生不存在，userId={}", orderGoodsInfo.getUserId());
                    throw new BizException("用户信息未找到");
                }
                TenantInfo tenantInfo = tenantInfoService.detail(student.getTenantId());
                if (tenantInfo == null || !tenantIdList.contains(tenantInfo.getId())) {
                    log.error("订单创建前检测，学生不在当前机构下，userId={},tenantId={}", orderGoodsInfo.getUserId(), student.getTenantId());
                    throw new BizException("用户不在当前机构");
                }
                price = tenantAlbum.getSalePrice();
                userOrderDetail.setMerchId(tenantInfo.getId());
                tenantAlbumContent.setBuyNumber(1);
                tenantAlbumContent.setBuyCycle(tenantAlbumContent.getBuyCycle()*tenantAlbumContent.getBuyMultiple());
                tenantAlbumContent.setTenantId(tenantInfo.getId());
                break;
            }

            case TENANT: {
                // 专辑是否在机构下
                TenantStaff tenantStaff = tenantStaffService.getByUserId(orderGoodsInfo.getUserId());
                if (tenantStaff == null) {
                    log.error("订单创建前检测，机构员工不存在，userId={}", orderGoodsInfo.getUserId());
                    throw new BizException("用户信息未找到");
                }
                if (!tenantIdList.contains(tenantStaff.getTenantId())) {
                    log.error("订单创建前检测，专辑不在机构下，userId={},tenantId={}", orderGoodsInfo.getUserId(), tenantStaff.getTenantId());
                    throw new BizException("用户不在当前机构");
                }
                price = tenantAlbum.getOriginalPrice();
                userOrderDetail.setMerchId(0L);
                tenantAlbumContent.setTenantId(tenantStaff.getTenantId());
                tenantAlbumContent.setBuyCycle(tenantAlbumContent.getBuyCycle()*tenantAlbumContent.getBuyMultiple());
                break;
            }
        }



        userOrderDetail.setGoodUrl(userOrderService.getGoodUrlByType(orderGoodsInfo.getGoodType()));
        userOrderDetail.setBizId(tenantAlbum.getId());
        userOrderDetail.setBizContent(JSON.toJSONString(tenantAlbumContent));
        userOrderDetail.setGoodNum(orderGoodsInfo.getGoodNum());
        userOrderDetail.setOriginalPrice(price.multiply(new BigDecimal(orderGoodsInfo.getGoodNum()))
                .multiply(new BigDecimal(tenantAlbumContent.getBuyMultiple())));
        userOrderDetail.setCouponAmount(BigDecimal.ZERO);
        userOrderDetail.setExpectPrice(userOrderDetail.getOriginalPrice());
        userOrderDetail.setActualPrice(userOrderDetail.getExpectPrice());

        orderGoodsInfo.setUserOrderDetail(userOrderDetail);


        // 设置金额入账去向
        UserPaymentOrderWrapper.AccountTenantTo accountTenantTo = new UserPaymentOrderWrapper.AccountTenantTo();
        if (orderGoodsInfo.getRecomUserId() !=null) {
            accountTenantTo.setShareTenant(teacherService.teacherSettlementFrom(orderGoodsInfo.getRecomUserId()));
        }

        // 存入缓存
        redisCacheService.saveUserAccountConfig(orderGoodsInfo.getSubOrderNo(), accountTenantTo.jsonString());

    }

    /**
     * 订单完成后
     *
     * @param userOrderDetailVo
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void orderSuccess(UserOrderDetailVo userOrderDetailVo) {

        // 学生 插入机构专辑购买数据
        // 学生购买机构专辑
        switch (userOrderDetailVo.getOrderClient()) {
            case STUDENT: {
                tenantActivationCodeService.addUserTenantAlbumRecord(userOrderDetailVo.getUserId(),userOrderDetailVo);

                break;
            }

            case TENANT: {
                // 机构   插入机构专辑购买记录，生成机构专辑激活码

                TenantAlbumWrapper.TenantAlbumContent tenantAlbumContent = JSON
                        .parseObject(userOrderDetailVo.getBizContent(), TenantAlbumWrapper.TenantAlbumContent.class);
                TenantAlbumPurchase tenantAlbumPurchase = new TenantAlbumPurchase();
                tenantAlbumPurchase.setTenantId(tenantAlbumContent.getTenantId());
                tenantAlbumPurchase.setTenantAlbumId(userOrderDetailVo.getBizId());
                tenantAlbumPurchase.setPurchaseTime(userOrderDetailVo.getCreateTime());
                tenantAlbumPurchase.setPurchaseQuantity(userOrderDetailVo.getGoodNum());
                tenantAlbumPurchase.setPurchaseCycle(tenantAlbumContent.getBuyCycle());
                tenantAlbumPurchase.setActiveQuantity(0);
                tenantAlbumPurchase.setOrderNo(userOrderDetailVo.getOrderNo());
                tenantAlbumPurchase.setPurchasePrice(userOrderDetailVo.getActualPrice());
                tenantAlbumPurchase.setPurchaseStatus(OrderStatusEnum.PAID.getCode());
                tenantAlbumPurchaseService.save(tenantAlbumPurchase);

                List<TenantActivationCode> tenantActivationCodes = new ArrayList<>();
                for (int i = 0; i < userOrderDetailVo.getGoodNum(); i++) {
                    TenantActivationCode tenantActivationCode = new TenantActivationCode();
                    tenantActivationCode.setTenantId(tenantAlbumContent.getTenantId());
                    tenantActivationCode.setTenantAlbumId(tenantAlbumContent.getTenantAlbumId());
                    tenantActivationCode.setTenantAlbumPurchaseId(tenantAlbumPurchase.getId());
                    long l = Long.parseLong(new Date().getTime() + i + String.valueOf(tenantAlbumPurchase.getId()));
                    tenantActivationCode.setActivationCode(StringUtil.DeciamlToThirtySix(l,7));
//                    tenantActivationCode.setActivationCode(IdWorker.get32UUID());
                    tenantActivationCode.setSendStatus(EActivationCode.WAIT);
                    tenantActivationCodes.add(tenantActivationCode);
                }
                tenantActivationCodeService.saveBatch(tenantActivationCodes);
                break;
            }
        }
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void insertTenantAlbum(Long tenantId, TenantAlbum tenantAlbum,
                                  List<TenantAlbumWrapper.MusicSheetData> musicSheetDataList) {

        List<String> subjectTypeNames = musicSheetDataList.stream().map(next -> next.getSubjectType().name())
                .distinct().collect(Collectors.toList());
        tenantAlbum.setSubjectTypes(String.join(",", subjectTypeNames));
        //给机构专辑赋值
        tenantAlbum.setStatus(false);
        tenantAlbum.setDelFlag(false);
        this.add(tenantAlbum);

        //关联表赋值
        TenantAlbumRef ref = new TenantAlbumRef();
        ref.setTenantAlbumId(tenantAlbum.getId());
        ref.setTenantId(tenantId);
        tenantAlbumRefService.add(ref);

        //曲目表赋值
        insertAlbumMusic(tenantId, tenantAlbum, musicSheetDataList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateAlbum(Long tenantId, TenantAlbum tenantAlbum,
                            List<TenantAlbumWrapper.MusicSheetData> musicSheetDataList) {

        List<String> subjectTypeNames = musicSheetDataList.stream().map(next -> next.getSubjectType().name())
                .distinct().collect(Collectors.toList());
        tenantAlbum.setSubjectTypes(String.join(",", subjectTypeNames));
        this.updateById(tenantAlbum);

        tenantAlbumMusicService.lambdaUpdate()
                .set(TenantAlbumMusic::getDelFlag, true)
                .eq(TenantAlbumMusic::getTenantAlbumId, tenantAlbum.getId())
                .eq(TenantAlbumMusic::getTenantId, tenantId)
                .update();

        insertAlbumMusic(tenantId, tenantAlbum, musicSheetDataList);

    }

    @Transactional(rollbackFor = Exception.class)
    private void insertAlbumMusic(Long tenantId, TenantAlbum tenantAlbum,
                                  List<TenantAlbumWrapper.MusicSheetData> musicSheetDataList) {
        //曲目表赋值
        List<TenantAlbumMusic> tenantAlbumMusics = musicSheetDataList.stream().map(next -> {
            AtomicInteger sort = new AtomicInteger(1);
            return next.getTenantAlbumSheetList().stream().map(tenantAlbumMusic -> {
                TenantAlbumMusic tenantAlbumMusic1 = new TenantAlbumMusic();
                tenantAlbumMusic1.setTenantId(tenantId);
                tenantAlbumMusic1.setSubjectType(next.getSubjectType());
                tenantAlbumMusic1.setTenantAlbumId(tenantAlbum.getId());
                tenantAlbumMusic1.setMusicSheetId(Long.valueOf(tenantAlbumMusic.getId()));
                tenantAlbumMusic1.setSortNumber(sort.getAndIncrement());
                tenantAlbumMusic1.setDelFlag(false);
                tenantAlbumMusic1.setLevel(tenantAlbumMusic.getLevel());
                tenantAlbumMusic1.setType(tenantAlbumMusic.getType());
                return tenantAlbumMusic1;
            }).collect(Collectors.toList());
        }).flatMap(Collection::stream).collect(Collectors.toList());

        if (!tenantAlbumMusics.isEmpty()) {
            tenantAlbumMusicService.saveBatch(tenantAlbumMusics, 100);
        }
    }

}